Attribute VB_Name = "MarchingSquare"
Option Explicit
Private Function Interpolace2(a As Single, b As Single, Limit As Single) As Single
    ' interpolace na stred hrany ( v rozmezi 0-1)
    Interpolace2 = (a - Limit) / (a - b)
    'Interpolace2 = 0.5
    
End Function
Private Function CheckLimit2(Limit As Single, Value As Single) As Integer
' kontrola zda je roh krychle vetsi nez limit
If Value > Limit Then
    CheckLimit2 = 1
    Else: CheckLimit2 = 0
End If
End Function

Public Sub GenerateContour(VoxelDescription, Voxel, Limit As Single, _
ZHight As Integer, Layer As Integer, LayerLines, NumberOfLines As Long)

' --------------------
'HLAVNI VSTUPNI FUNKCE'
'---------------------
' marhing Sguare - ekvivalent of MarchingCube for 2D maping
'VoxelDescription 3x Rozmery voxelu, x0,y0,z0,krokx,kroky,krokz
'Voxel - pole x,y,z s prostorovou hodnotou
'Limit - limit vyberu pro kladnou intenzitu
'LayerLines - pole pro ulozeni koncovych a pocatecnich bodu linek
'fomat NuberOfLines,2,3 pro x,y,z koncovych bodu linek
'NumberOfLines - pocit linek ve vyse uvedenem

Dim SquareValues(1 To 4) As Single
Dim SquarePoints(1 To 4) As Integer
Dim X As Integer
Dim Y As Integer
Dim index As Integer
Dim i As Integer

' ctverec , cislovani vrcholu a hran
'                 3
'             4------3
'             |      |
'            4|      |2
'             |      |
'             1------2
'                 1

NumberOfLines = 0

For X = 1 To VoxelDescription(1) - 1 'rez
  For Y = 1 To VoxelDescription(2) - 1
        
    SquareValues(1) = Voxel(X, Y, ZHight)
    SquareValues(2) = Voxel(X + 1, Y, ZHight)
    SquareValues(3) = Voxel(X + 1, Y + 1, ZHight)
    SquareValues(4) = Voxel(X, Y + 1, ZHight)
            
      For i = 1 To 4
        SquarePoints(i) = CheckLimit2(Limit, SquareValues(i))
      Next i
                        
            ' Vypocet indexu pro urceni topologie krychle
    index = SquarePoints(1) + _
             SquarePoints(2) * 2 + _
             SquarePoints(3) * 4 + _
             SquarePoints(4) * 8

    If index = 0 Or index = 15 Then GoTo skip ' zadny nebo zcela zaplneni prostor
            
    GenerateSquareLines index, _
    (((X - 1) * VoxelDescription(7)) + VoxelDescription(4)), _
    (((Y - 1) * VoxelDescription(8)) + VoxelDescription(5)), _
    (((ZHight - 1) * VoxelDescription(9)) + VoxelDescription(6)), _
    SquareValues, _
    Layer, LayerLines, NumberOfLines, _
    Limit, VoxelDescription
skip:
    Next Y
Next X
        
End Sub

Private Sub GenerateSquareLines(index As Integer, _
x0 As Single, y0 As Single, z0 As Single, _
SquareValues, Layer As Integer, LayerLines, _
NumberOfLines As Long, Limit As Single, VoxelDescription)
 
Dim NewLines As Integer
Dim SqTop(1 To 2, 2) As Integer
' number of lines, end point on edge
Dim i As Integer
Dim j As Integer
Dim StartLine(1 To 3) As Single
Dim EndLine(1 To 3) As Single

Select Case index
  Case 1
    NewLines = 1
    SqTop(1, 1) = 1
    SqTop(1, 2) = 4
  Case 2
    NewLines = 1
    SqTop(1, 1) = 1
    SqTop(1, 2) = 2
  Case 3
    NewLines = 1
    SqTop(1, 1) = 4
    SqTop(1, 2) = 2
  Case 4
    NewLines = 1
    SqTop(1, 1) = 2
    SqTop(1, 2) = 3
  Case 5
    NewLines = 2
    SqTop(1, 1) = 1
    SqTop(1, 2) = 4
    SqTop(2, 1) = 2
    SqTop(2, 2) = 3
  Case 6
    NewLines = 1
    SqTop(1, 1) = 1
    SqTop(1, 2) = 3
  Case 7
    NewLines = 1
    SqTop(1, 1) = 4
    SqTop(1, 2) = 3
  Case 8
    NewLines = 1
    SqTop(1, 1) = 4
    SqTop(1, 2) = 3
  Case 9
    NewLines = 1
    SqTop(1, 1) = 1
    SqTop(1, 2) = 3
  Case 10
    NewLines = 2
    SqTop(1, 1) = 1
    SqTop(1, 2) = 2
    SqTop(2, 1) = 3
    SqTop(2, 2) = 4
  Case 11
    NewLines = 1
    SqTop(1, 1) = 2
    SqTop(1, 2) = 3
  Case 12
    NewLines = 1
    SqTop(1, 1) = 2
    SqTop(1, 2) = 4
  Case 13
    NewLines = 1
    SqTop(1, 1) = 1
    SqTop(1, 2) = 2
  Case 14
    NewLines = 1
    SqTop(1, 1) = 1
    SqTop(1, 2) = 4
  Case Else
    Stop
End Select

For i = 1 To NewLines
  MakePrusecik x0, y0, z0, SquareValues, VoxelDescription, Limit, StartLine, SqTop(i, 1)
  MakePrusecik x0, y0, z0, SquareValues, VoxelDescription, Limit, EndLine, SqTop(i, 2)
  NumberOfLines = NumberOfLines + 1
    For j = 1 To 3
      LayerLines(Layer, NumberOfLines, 1, j) = StartLine(j)
      LayerLines(Layer, NumberOfLines, 2, j) = EndLine(j)
    Next j
Next i

End Sub
Private Sub MakePrusecik(x0 As Single, y0 As Single, z0 As Single, _
SquareValues, VoxelDescription, Limit As Single, _
Position, edge As Integer)

Dim v1 As Single
Dim v2 As Single
Dim v3 As Single
Dim v4 As Single
Dim Krokx As Single
Dim Kroky As Single
v1 = SquareValues(1)
v2 = SquareValues(2)
v3 = SquareValues(3)
v4 = SquareValues(4)
Krokx = VoxelDescription(7)
Kroky = VoxelDescription(8)


Position(3) = z0

Select Case edge
  Case 1
    Position(1) = x0 + Krokx * Interpolace2(v1, v2, Limit)
    Position(2) = y0
  Case 2
    Position(1) = x0 + Krokx
    Position(2) = y0 + Kroky * Interpolace2(v2, v3, Limit)
  Case 3
    Position(1) = x0 + Krokx * Interpolace2(v4, v3, Limit)
    Position(2) = y0 + Kroky
  Case 4
    Position(1) = x0
    Position(2) = y0 + Kroky * Interpolace2(v1, v4, Limit)
  Case Else
    Stop
End Select

End Sub

